@@ -17,7 +17,6 @@ Post from patricknik on 5 Jan re ""b"" in a field. And Aykut Firat on email.
 Save repeated ch<eof checking in main read step. Last line might still be tricky if last line has no eol.
 Test using at least "grep read.table ...Rtrunk/tests/
 Look for any non-alpha-numeric characters in the output and try each of them. That way can handle bell character as well and save testing separators which aren't there.
-Column all 0 and 1 treated as logical?
 Just one "NA" current default but empty in future when numerics handle NA string variants directly.
 ---
 Secondary separator for list() columns, such as columns 11 and 12 in BED (no need for strsplit).
@@ -31,15 +30,14 @@ static int  typeEnum[NUT] =    {CT_DROP, CT_BOOL8_N, CT_BOOL8_U, CT_BOOL8_T, CT_
 static colType readInt64As=CT_INT64;
 static SEXP selectSxp;
 static SEXP dropSxp;
-static SEXP colNamesSxp;
 static SEXP colClassesSxp;
 static cetype_t ienc = CE_NATIVE;
 static SEXP DT;
+static SEXP colNamesSxp;
 static int8_t *type;
 static int8_t *size;
 static int ncol = 0;
 static int64_t dtnrows = 0;
-static int protecti=0;
 static _Bool verbose = 0;
 static _Bool warningsAreErrors = 0;
 
@@ -74,7 +72,6 @@ SEXP freadR(
   warningsAreErrors = LOGICAL(warnings2errorsArg)[0];
 
   freadMainArgs args;
-  protecti=0;
   ncol = 0;
   dtnrows = 0;
   const char *ch, *ch2;
@@ -178,11 +175,11 @@ SEXP freadR(
 
   DT = R_NilValue; // created by callback
   freadMain(args);
-  UNPROTECT(protecti);
+  if (DT!=R_NilValue) UNPROTECT(1);  // DT is allocated in allocateDT. See notes there.
+  if (colNamesSxp!=R_NilValue) UNPROTECT(1);  // allocated in userOverride() and then used in allocateDT()
   return DT;
 }
 
-
 _Bool userOverride(int8_t *type, lenOff *colNames, const char *anchor, int ncol)
 {
   // use typeSize superfluously to avoid not-used warning; otherwise could move typeSize from fread.h into fread.c
@@ -191,7 +188,7 @@ _Bool userOverride(int8_t *type, lenOff *colNames, const char *anchor, int ncol)
   colNamesSxp = R_NilValue;
   if (colNames!=NULL) {
     colNamesSxp = PROTECT(allocVector(STRSXP, ncol));
-    protecti++;
+    // no protecti++ and no UNPROTECT inside userOverride because it's used in allocateDT().
     for (int i=0; i<ncol; i++) {
       SEXP this;
       if (colNames[i].len<=0) {
@@ -222,7 +219,7 @@ _Bool userOverride(int8_t *type, lenOff *colNames, const char *anchor, int ncol)
         STOP("colClasses is an unnamed character vector but its length is %d. Must be length 1 or ncol (%d in this case) when unnamed. To specify types for a subset of columns you can either name the items with the column names or pass list() format to colClasses using column names or column numbers. See examples in ?fread.",
               LENGTH(colClassesSxp), ncol);
       }
-      UNPROTECT(1);  // typeEnum_idx
+      UNPROTECT(1); // typeEnum_idx
     } else {
       if (!isNewList(colClassesSxp)) STOP("CfreadR: colClasses is not type list");
       if (!length(getAttrib(colClassesSxp, R_NamesSymbol))) STOP("CfreadR: colClasses is type list but has no names");
@@ -238,7 +235,8 @@ _Bool userOverride(int8_t *type, lenOff *colNames, const char *anchor, int ncol)
         }
         SEXP itemsInt;
         if (isString(items)) itemsInt = PROTECT(chmatch(items, colNamesSxp, NA_INTEGER, FALSE));
-        else itemsInt = PROTECT(coerceVector(items, INTSXP));
+        else                 itemsInt = PROTECT(coerceVector(items, INTSXP));
+        // UNPROTECTed directly just after this for loop. No protecti++ here is correct.
         for (int j=0; j<LENGTH(items); j++) {
           int k = INTEGER(itemsInt)[j];
           if (k==NA_INTEGER) {
@@ -252,12 +250,12 @@ _Bool userOverride(int8_t *type, lenOff *colNames, const char *anchor, int ncol)
             // freadMain checks bump up only not down.  Deliberately don't catch here to test freadMain; e.g. test 959
           }
         }
-        UNPROTECT(1); // itemsInt
+        UNPROTECT(1); // UNPROTECTing itemsInt inside loop to save protection stack
       }
       for (int i=0; i<ncol; i++) if (type[i]<0) type[i] *= -1;  // undo sign; was used to detect duplicates
       UNPROTECT(1);  // typeEnum_idx
     }
-    UNPROTECT(1); // typeRName_sxp
+    UNPROTECT(1);  // typeRName_sxp
   }
   if (readInt64As != CT_INT64) {
     for (int i=0; i<ncol; i++) if (type[i]==CT_INT64) type[i] = readInt64As;
@@ -265,7 +263,7 @@ _Bool userOverride(int8_t *type, lenOff *colNames, const char *anchor, int ncol)
   if (length(dropSxp)) {
     SEXP itemsInt;
     if (isString(dropSxp)) itemsInt = PROTECT(chmatch(dropSxp, colNamesSxp, NA_INTEGER, FALSE));
-    else itemsInt = PROTECT(coerceVector(dropSxp, INTSXP));
+    else                   itemsInt = PROTECT(coerceVector(dropSxp, INTSXP));
     for (int j=0; j<LENGTH(itemsInt); j++) {
       int k = INTEGER(itemsInt)[j];
       if (k==NA_INTEGER) {
@@ -286,14 +284,14 @@ _Bool userOverride(int8_t *type, lenOff *colNames, const char *anchor, int ncol)
     UNPROTECT(1); // itemsInt
   } else if (length(selectSxp)) {
     SEXP tt;
-    int nprot = 0;
     if (isString(selectSxp)) {
       // invalid cols check part of #1445 moved here (makes sense before reading the file)
       tt = PROTECT(chmatch(selectSxp, colNamesSxp, NA_INTEGER, FALSE));
-      nprot++;
       for (int i=0; i<length(selectSxp); i++) if (INTEGER(tt)[i]==NA_INTEGER)
         DTWARN("Column name '%s' not found in column name header (case sensitive), skipping.", CHAR(STRING_ELT(selectSxp, i)));
-    } else tt = selectSxp;
+    } else {
+      tt = PROTECT(selectSxp); // harmless superfluous PROTECT, for ease of balancing
+    }
     for (int i=0; i<LENGTH(tt); i++) {
       int k = isInteger(tt) ? INTEGER(tt)[i] : (int)REAL(tt)[i];
       if (k == NA_INTEGER) continue;
@@ -307,7 +305,7 @@ _Bool userOverride(int8_t *type, lenOff *colNames, const char *anchor, int ncol)
       if (type[i]<0) type[i] *= -1;
       else type[i]=CT_DROP;
     }
-    UNPROTECT(nprot);
+    UNPROTECT(1); // tt
   }
   return true;
 }
@@ -321,8 +319,9 @@ size_t allocateDT(int8_t *typeArg, int8_t *sizeArg, int ncolArg, int ndrop, size
   if (newDT) {
     ncol = ncolArg;
     dtnrows = allocNrow;
-    DT=PROTECT(allocVector(VECSXP,ncol-ndrop));  // safer to leave over allocation to alloc.col on return in fread.R
-    protecti++;
+    DT = PROTECT(allocVector(VECSXP,ncol-ndrop));
+    // no protecti++ here. See notes at the end of this function.
+    // over-allocation is left to alloc.col on return in fread.R
     if (ndrop==0) {
       setAttrib(DT,R_NamesSymbol,colNamesSxp);  // colNames mkChar'd in userOverride step
     } else {
@@ -367,6 +366,15 @@ size_t allocateDT(int8_t *typeArg, int8_t *sizeArg, int ncolArg, int ndrop, size
     resi++;
   }
   dtnrows = allocNrow;
+
+  // rchk on CRAN check page will note there is no UNPROTECT here. This is correct.
+  // ------------------------------------------------------------------------------
+  // This allocateDT function is called from freadMean. freadMain is in fread.c which is a C file independent
+  // of R (it is shared with pydatatable) and contains no R-API calls. freadMain opens the csv file, detects
+  // ncols and types, calls back to this allocateDT function and then continues to populate DT before
+  // returning back to freadR higher up in this file, which is where the UNPROTECT happens.
+  // NB: rchk will note this on CRAN check page
+
   return DTbytes;
 }
 
